@@ -40,6 +40,8 @@
 #include <unistd.h>
 #include <fcntl.h>
 #include <signal.h>
+#include <glib.h>
+#include <glib/gstdio.h>
 #include <libgnomevfs/gnome-vfs.h>
 #include <gtk/gtkmessagedialog.h>
 
@@ -262,6 +264,7 @@ get_backing_file (const char *virtual_path)
 	escaped_path = gnome_vfs_escape_path_string (virtual_path);
 	uri = g_strconcat ("burn:///", escaped_path, NULL);
 	g_free (escaped_path);
+	/* warning, this can hang on fifos etc */
 	res = gnome_vfs_open (&handle,
 			      uri,
 			      GNOME_VFS_OPEN_READ);
@@ -327,6 +330,25 @@ graft_file_end_dir_visitor (struct mkisofs_state *state)
 	}
 }
 
+/* FIXME: This should probably be an inline */
+static gboolean
+file_info_is_allowed (GnomeVFSFileInfo *info)
+{
+	g_return_val_if_fail (info != NULL, FALSE);
+
+	/* only allow regular,directory,symlink files */
+	if (info->type != GNOME_VFS_FILE_TYPE_REGULAR
+	    && info->type != GNOME_VFS_FILE_TYPE_DIRECTORY
+	    && info->type != GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK) {
+		return FALSE;
+	} else if (info->type == GNOME_VFS_FILE_TYPE_REGULAR
+		   && !(info->permissions & GNOME_VFS_PERM_ACCESS_READABLE)) {
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static gboolean
 graft_file_visitor (const gchar          *rel_path,
 		    GnomeVFSFileInfo     *info,
@@ -339,6 +361,11 @@ graft_file_visitor (const gchar          *rel_path,
 
 	*recurse = TRUE;
 	
+	if (! file_info_is_allowed (info)) {
+		g_warning ("Skipping file: %s", rel_path);
+		return TRUE;
+	}
+
 	if (state->copy_to_dir != NULL) {
 		if (info->type == GNOME_VFS_FILE_TYPE_DIRECTORY) {
 			new_copy_dir = g_build_filename (state->copy_to_dir,
@@ -413,7 +440,10 @@ create_graft_file (GnomeVFSURI          *uri,
 	GnomeVFSResult           result;
 	gboolean                 stop;
 
-	result = gnome_vfs_directory_open_from_uri (&handle, uri, GNOME_VFS_FILE_INFO_DEFAULT);
+	result = gnome_vfs_directory_open_from_uri (&handle,
+						    uri,
+						    GNOME_VFS_FILE_INFO_DEFAULT
+						    | GNOME_VFS_FILE_INFO_GET_ACCESS_RIGHTS);
 	if (result != GNOME_VFS_OK)
 		return;
 
@@ -501,6 +531,10 @@ mkisofs_stdout_read (GIOChannel   *source,
 
 		if (status == G_IO_STATUS_NORMAL) {
 			g_free (line);
+		} else if (status == G_IO_STATUS_EOF) {
+			if (mkisofs_output->debug)
+				g_print ("make_iso stdout: EOF\n");
+			return FALSE;
 		}
 	} else if (condition & G_IO_HUP) {
 		if (mkisofs_output->debug)
@@ -583,6 +617,12 @@ mkisofs_stderr_read (GIOChannel   *source,
 			}
 
 			g_free (line);
+		} else if (status == G_IO_STATUS_EOF) {
+			if (mkisofs_output->debug)
+				g_print ("make_iso stderr: EOF\n");
+			nautilus_burn_progress_set_fraction (1.0);
+			g_main_loop_quit (mkisofs_output->loop);
+			return FALSE;
 		}
 	} else if (condition & G_IO_HUP) {
 		/* only handle the HUP when we have read all available lines of output */
@@ -899,6 +939,8 @@ nautilus_burn_make_iso (const char *filename,
 								    NAUTILUS_BURN_ISO_ERROR_GENERAL,
 								    _("Unknown error"));
 
+		g_unlink (mkisofs_output.filename);
+
 		g_propagate_error (error, mkisofs_output.error);
 	}
 
