@@ -463,6 +463,25 @@ typedef struct
 	gboolean	 only_isos;
 } EstimateSizeCallbackData;
 
+/* FIXME: This should probably be an inline */
+static gboolean
+file_info_is_allowed (GnomeVFSFileInfo *info)
+{
+	g_return_val_if_fail (info != NULL, FALSE);
+
+	/* only allow regular,directory,symlink files */
+	if (info->type != GNOME_VFS_FILE_TYPE_REGULAR
+	    && info->type != GNOME_VFS_FILE_TYPE_DIRECTORY
+	    && info->type != GNOME_VFS_FILE_TYPE_SYMBOLIC_LINK) {
+		return FALSE;
+	} else if (info->type == GNOME_VFS_FILE_TYPE_REGULAR
+		   && !(info->permissions & GNOME_VFS_PERM_ACCESS_READABLE)) {
+		return FALSE;
+	}
+
+	return TRUE;
+}
+
 static gboolean
 estimate_size_callback (const gchar      *rel_path,
 			GnomeVFSFileInfo *info,
@@ -476,12 +495,13 @@ estimate_size_callback (const gchar      *rel_path,
 	    && info->mime_type != NULL
 	    && strcmp (info->mime_type, "application/x-cd-image") != 0)
 		cb_data->only_isos = FALSE;
-	if (info->type != GNOME_VFS_FILE_TYPE_DIRECTORY
-	    && !(info->permissions & GNOME_VFS_PERM_ACCESS_READABLE))
-		cb_data->unreadable_paths = g_list_append (cb_data->unreadable_paths, g_strdup (rel_path));
 
-	if (info->type != GNOME_VFS_FILE_TYPE_DIRECTORY) {
-		cb_data->size += info->size;
+	if (file_info_is_allowed (info)) {
+		if (info->type != GNOME_VFS_FILE_TYPE_DIRECTORY) {
+			cb_data->size += info->size;
+		}
+	} else {
+		cb_data->unreadable_paths = g_list_append (cb_data->unreadable_paths, g_strdup (rel_path));
 	}
 
 	*recurse = TRUE;
@@ -754,7 +774,7 @@ burn_cd (const NautilusBurnDrive *drive,
 	 gboolean                 overburn,
 	 gboolean                 burnproof)
 {
-	NautilusBurnRecorderWriteFlags flags;
+	NautilusBurnRecorderWriteFlags flags = 0;
 	GList                         *tracks;
 	GError                        *error = NULL;
 	char                          *filename, *str;
